@@ -865,6 +865,7 @@ int snd_func_private_pcm_subdevice(snd_config_t **dst, snd_config_t *root ATTRIB
 {
 	snd_pcm_info_t *info;
 	const char *id;
+	void *data;
 	snd_pcm_t *pcm;
 	int err;
 
@@ -875,7 +876,8 @@ int snd_func_private_pcm_subdevice(snd_config_t **dst, snd_config_t *root ATTRIB
 		SNDERR("field pcm_handle not found");
 		return -EINVAL;
 	}
-	err = snd_config_get_pointer(private_data, (const void **)&pcm);
+	err = snd_config_get_pointer(private_data, &data);
+	pcm = data;
 	if (err < 0) {
 		SNDERR("field pcm_handle is not a pointer");
 		return err;
