@@ -396,28 +396,28 @@ static int snd_ctl_shm_read(snd_ctl_t *ctl, snd_ctl_event_t *event)
 }
 
 snd_ctl_ops_t snd_ctl_shm_ops = {
-	close: snd_ctl_shm_close,
-	nonblock: snd_ctl_shm_nonblock,
-	async: snd_ctl_shm_async,
-	subscribe_events: snd_ctl_shm_subscribe_events,
-	card_info: snd_ctl_shm_card_info,
-	element_list: snd_ctl_shm_elem_list,
-	element_info: snd_ctl_shm_elem_info,
-	element_read: snd_ctl_shm_elem_read,
-	element_write: snd_ctl_shm_elem_write,
-	element_lock: snd_ctl_shm_elem_lock,
-	element_unlock: snd_ctl_shm_elem_unlock,
-	hwdep_next_device: snd_ctl_shm_hwdep_next_device,
-	hwdep_info: snd_ctl_shm_hwdep_info,
-	pcm_next_device: snd_ctl_shm_pcm_next_device,
-	pcm_info: snd_ctl_shm_pcm_info,
-	pcm_prefer_subdevice: snd_ctl_shm_pcm_prefer_subdevice,
-	rawmidi_next_device: snd_ctl_shm_rawmidi_next_device,
-	rawmidi_info: snd_ctl_shm_rawmidi_info,
-	rawmidi_prefer_subdevice: snd_ctl_shm_rawmidi_prefer_subdevice,
-	set_power_state: snd_ctl_shm_set_power_state,
-	get_power_state: snd_ctl_shm_get_power_state,
-	read: snd_ctl_shm_read,
+	.close = snd_ctl_shm_close,
+	.nonblock = snd_ctl_shm_nonblock,
+	.async = snd_ctl_shm_async,
+	.subscribe_events = snd_ctl_shm_subscribe_events,
+	.card_info = snd_ctl_shm_card_info,
+	.element_list = snd_ctl_shm_elem_list,
+	.element_info = snd_ctl_shm_elem_info,
+	.element_read = snd_ctl_shm_elem_read,
+	.element_write = snd_ctl_shm_elem_write,
+	.element_lock = snd_ctl_shm_elem_lock,
+	.element_unlock = snd_ctl_shm_elem_unlock,
+	.hwdep_next_device = snd_ctl_shm_hwdep_next_device,
+	.hwdep_info = snd_ctl_shm_hwdep_info,
+	.pcm_next_device = snd_ctl_shm_pcm_next_device,
+	.pcm_info = snd_ctl_shm_pcm_info,
+	.pcm_prefer_subdevice = snd_ctl_shm_pcm_prefer_subdevice,
+	.rawmidi_next_device = snd_ctl_shm_rawmidi_next_device,
+	.rawmidi_info = snd_ctl_shm_rawmidi_info,
+	.rawmidi_prefer_subdevice = snd_ctl_shm_rawmidi_prefer_subdevice,
+	.set_power_state = snd_ctl_shm_set_power_state,
+	.get_power_state = snd_ctl_shm_get_power_state,
+	.read = snd_ctl_shm_read,
 };
 
 static int make_local_socket(const char *filename)
