@@ -30,6 +30,15 @@
  *
  */
 
+/*! \page mixer Mixer interface
+
+<P>Mixer interface is designed to access the abstracted mixer controls.
+This is an abstraction layer over the hcontrol layer.
+
+\section control_general_overview General overview
+
+*/
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
