@@ -1,4 +1,4 @@
-/*
+/**
  * \file mixer/simple.c
  * \brief Mixer Simple Element Class Interface
  * \author Jaroslav Kysela <perex@suse.cz>
@@ -110,7 +110,7 @@ static struct mixer_name_table {
 	{0, 0},
 };
 
-#endif
+#endif /* !DOC_HIDDEN */
 
 static const char *get_short_name(const char *lname)
 {
@@ -329,8 +329,12 @@ static int selem_read(snd_mixer_elem_t *elem)
 	csw = s->str[CAPT].sw;
 	s->str[CAPT].sw = ~0U;
 
-	if (s->ctls[CTL_ENUMLIST].elem)
-		return elem_read_enum(s);
+	if (s->ctls[CTL_ENUMLIST].elem) {
+		err = elem_read_enum(s);
+		if (err < 0)
+			return err;
+		goto __skip_cswitch;
+	}
 
 	if (s->ctls[CTL_PLAYBACK_VOLUME].elem)
 		err = elem_read_volume(s, PLAY, CTL_PLAYBACK_VOLUME);
@@ -1104,7 +1108,8 @@ static int simple_compare(const snd_mixer_elem_t *c1, const snd_mixer_elem_t *c2
  * \param classp Pointer to returned mixer simple element class handle (or NULL)
  * \return 0 on success otherwise a negative error code
  */
-int snd_mixer_selem_register(snd_mixer_t *mixer, void *options ATTRIBUTE_UNUSED,
+int snd_mixer_selem_register(snd_mixer_t *mixer, struct
+			     snd_mixer_selem_regopt *options ATTRIBUTE_UNUSED,
 			     snd_mixer_class_t **classp)
 {
 	snd_mixer_class_t *class = calloc(1, sizeof(*class));
