@@ -898,7 +898,7 @@ int snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 			return ret;
 		}
 			
-		ret = snd_pcm_hw_open_fd(&spcm, "dmix_client", dmix->hw_fd, 0);
+		ret = snd_pcm_hw_open_fd(&spcm, "dmix_client", dmix->hw_fd, 0, 0);
 		if (ret < 0) {
 			SNDERR("unable to open hardware");
 			goto _err;
@@ -991,6 +991,7 @@ pcm.name {
 	type dmix		# Direct mix
 	ipc_key INT		# unique IPC key
 	ipc_key_add_uid BOOL	# add current uid to unique IPC key
+	ipc_perm INT		# IPC permissions (octal, default 0600)
 	slave STR
 	# or
 	slave {			# Slave definition
@@ -1011,6 +1012,7 @@ pcm.name {
 	bindings {		# note: this is client independent!!!
 		N INT		# maps slave channel to client channel N
 	}
+	slowptr BOOL		# slow but more precise pointer updates
 }
 \endcode
 
