@@ -495,6 +495,7 @@ static snd_pcm_fast_ops_t snd_pcm_dshare_fast_ops = {
 	.rewind = snd_pcm_dshare_rewind,
 	.forward = snd_pcm_dshare_forward,
 	.resume = snd_pcm_dshare_resume,
+	.poll_ask = NULL,
 	.writei = snd_pcm_mmap_writei,
 	.writen = snd_pcm_mmap_writen,
 	.readi = snd_pcm_dshare_readi,
@@ -631,7 +632,7 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 			return ret;
 		}
 			
-		ret = snd_pcm_hw_open_fd(&spcm, "dshare_client", dshare->hw_fd, 0);
+		ret = snd_pcm_hw_open_fd(&spcm, "dshare_client", dshare->hw_fd, 0, 0);
 		if (ret < 0) {
 			SNDERR("unable to open hardware");
 			goto _err;
@@ -726,6 +727,7 @@ pcm.name {
 	type dshare		# Direct sharing
 	ipc_key INT		# unique IPC key
 	ipc_key_add_uid BOOL	# add current uid to unique IPC key
+	ipc_perm INT		# IPC permissions (octal, default 0600)
 	slave STR
 	# or
 	slave {			# Slave definition
@@ -746,6 +748,7 @@ pcm.name {
 	bindings {		# note: this is client independent!!!
 		N INT		# maps slave channel to client channel N
 	}
+	slowptr BOOL		# slow but more precise pointer updates
 }
 \endcode
 
