@@ -275,6 +275,8 @@ static snd_pcm_sframes_t snd_pcm_null_avail_update(snd_pcm_t *pcm)
 static int snd_pcm_null_hw_refine(snd_pcm_t *pcm ATTRIBUTE_UNUSED, snd_pcm_hw_params_t *params)
 {
 	int err = snd_pcm_hw_refine_soft(pcm, params);
+	params->info = SND_PCM_INFO_MMAP | SND_PCM_INFO_MMAP_VALID |
+		       SND_PCM_INFO_RESUME | SND_PCM_INFO_PAUSE;
 	params->fifo_size = 0;
 	return err;
 }
@@ -342,6 +344,7 @@ static snd_pcm_fast_ops_t snd_pcm_null_fast_ops = {
 	.rewind = snd_pcm_null_rewind,
 	.forward = snd_pcm_null_forward,
 	.resume = snd_pcm_null_resume,
+	.poll_ask = NULL,
 	.writei = snd_pcm_null_writei,
 	.writen = snd_pcm_null_writen,
 	.readi = snd_pcm_null_readi,
