@@ -33,8 +33,6 @@
 #include <sched.h>
 #include <errno.h>
 #include <getopt.h>
-#define ALSA_PCM_NEW_HW_PARAMS_API
-#define ALSA_PCM_NEW_SW_PARAMS_API
 #include "../include/asoundlib.h"
 #include <sys/time.h>
 #include <math.h>
